/* This file is part of the KDE project
   Copyright (C) 2004 Arend van Beelen jr. <arend@auton.nl>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include <unistd.h>

#include <dcopclient.h>
#include <kapplication.h>
#include <kaction.h>
#include <kconfig.h>
#include <kdebug.h>
#include <kdesktopfile.h>
#include <kgenericfactory.h>
#include <kglobal.h>
#include <khtml_part.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kmimetype.h>
#include <kprocess.h>
#include <kprotocolmanager.h>
#include <kstandarddirs.h>
#include <kurifilter.h>

#include <qpopupmenu.h>
#include <qwhatsthis.h>
#include "searchbar.h"

typedef KGenericFactory<SearchBarPlugin> SearchBarPluginFactory;
K_EXPORT_COMPONENT_FACTORY(libsearchbarplugin,
                           SearchBarPluginFactory("searchbarplugin"))


SearchBarPlugin::SearchBarPlugin(QObject *parent, const char *name,
                                 const QStringList &) :
  KParts::Plugin(parent, name),
  m_searchCombo(0),
  m_searchMode(UseSearchProvider),
  m_urlEnterLock(false)
{
	m_part = static_cast<KHTMLPart *>(parent);

	m_searchCombo = new SearchBarCombo(0L, "search combo");
	m_searchCombo->setDuplicatesEnabled(false);
	m_searchCombo->setMaxCount(5);
	m_searchCombo->setMinimumWidth(180);

	m_popupMenu = new QPopupMenu(m_searchCombo, "search selection menu");
	m_popupMenu->insertItem(SmallIcon("find"), i18n("Find in This Page"), this, SLOT(useFindInThisPage()),  0, 0);
	m_popupMenu->insertSeparator();
	m_popupMenu->insertItem("", this, SLOT(useSearchProvider()),  0, 1);
	m_popupMenu->insertItem(SmallIcon("enhanced_browsing"), i18n("Select Search Engine..."),
	                        this, SLOT(selectSearchEngine()), 0, 2);

	KWidgetAction *searchComboAction = new KWidgetAction(m_searchCombo, i18n("Search Bar"), 0,
	                                                     0, 0, actionCollection(), "toolbar_search_bar");
	searchComboAction->setShortcutConfigurable(false);

	connect(m_searchCombo, SIGNAL(activated(const QString &)),
	                       SLOT(startSearch(const QString &)));
	connect(m_searchCombo, SIGNAL(iconClicked()), SLOT(showSelectionMenu()));

	QWhatsThis::add(m_searchCombo, i18n("Search Bar<p>"
	                                    "Enter a search term."));

	configurationChanged();
}

SearchBarPlugin::~SearchBarPlugin()
{
	KConfig *config = kapp->config();
	config->setGroup("SearchBar");
	config->writeEntry("Mode", (int) m_searchMode);

	delete m_searchCombo;
	m_searchCombo = 0L;
}

void SearchBarPlugin::startSearch(const QString &search)
{
	if(m_urlEnterLock || search.isEmpty())
		return;

	if(m_searchMode == FindInThisPage)
	{
		m_part->findText(search, 0);
		m_part->findTextNext();
	}
	else if(m_searchMode == UseSearchProvider)
	{
		m_urlEnterLock = true;
		KURIFilterData data;
		data.setData(search);
		QStringList list;
		list << "kurisearchfilter" << "kuriikwsfilter";

		if(KURIFilter::self()->filterURI(data, list))
		{
			emit m_part->browserExtension()->openURLRequest(data.uri());
		}
	}

	if(m_searchCombo->text(0).isEmpty())
	{
		m_searchCombo->changeItem(m_searchIcon, search, 0);
	}
	else
	{
		m_searchCombo->insertItem(m_searchIcon, search);
	}

	m_urlEnterLock = false;
}

void SearchBarPlugin::setIcon()
{
	QPixmap icon;
	KURIFilterData data;
	data.setData(QString("some keyword"));
	QStringList list;
	list << "kurisearchfilter" << "kuriikwsfilter";

	if(KURIFilter::self()->filterURI(data, list))
	{
		QString iconPath = locate("cache", KMimeType::favIconForURL(data.uri()) + ".png");
		if(iconPath.isEmpty())
		{
			icon = SmallIcon("find");
		}
		else
		{
			icon = QPixmap(iconPath);
		}
	}
	else
	{
		icon = SmallIcon("find");
	}
	m_popupMenu->changeItem(1, icon, m_providerName);

	m_searchIcon = (m_searchMode == FindInThisPage ? SmallIcon("find") : icon);

	m_searchCombo->insertItem(m_searchIcon, 0);
	for(int i = 1; i < m_searchCombo->count(); i++)
	{
		m_searchCombo->changeItem(m_searchIcon, m_searchCombo->text(i), i);
	}
}

void SearchBarPlugin::showSelectionMenu()
{
	m_popupMenu->popup(m_searchCombo->mapToGlobal(QPoint(0, m_searchCombo->height() + 1)), 0);
}

void SearchBarPlugin::useFindInThisPage()
{
	m_searchMode = FindInThisPage;
	setIcon();
}

void SearchBarPlugin::useSearchProvider()
{
	m_searchMode = UseSearchProvider;
	setIcon();
}

void SearchBarPlugin::selectSearchEngine()
{
	KProcess *process = new KProcess;

	*process << "kcmshell" << "ebrowsing";

	connect(process, SIGNAL(processExited(KProcess *)), SLOT(searchEngineSelected(KProcess *)));

	if(!process->start())
	{
		kdDebug(1202) << "Couldn't invoke kcmshell." << endl;
                delete process;
	}
}

void SearchBarPlugin::searchEngineSelected(KProcess *process)
{
	if(!process || process->exitStatus() == 0)
	{
		configurationChanged();
	}
        delete process;
}

void SearchBarPlugin::configurationChanged()
{
	KConfig *config = new KConfig("kuriikwsfilterrc");
	config->setGroup("General");
	QString engine = config->readEntry("DefaultSearchEngine", "google");
	delete config;
	KDesktopFile file("searchproviders/" + engine + ".desktop", true, "services");
	m_providerName = file.readName();

	config = kapp->config();
	config->setGroup("SearchBar");
	m_searchMode = (SearchModes) config->readNumEntry("Mode", (int) UseSearchProvider);

	setIcon();
}

SearchBarCombo::SearchBarCombo(QWidget *parent, const char *name) :
  KHistoryCombo(parent, name)
{
}

void SearchBarCombo::mousePressEvent(QMouseEvent *e)
{
	if(e->x() < pixmap(0)->width() + 6)
	{
		emit iconClicked();

		e->accept();
	}
	else
	{
		KHistoryCombo::mousePressEvent(e);
	}
}

#include "searchbar.moc"
